@@ -1,5 +1,6 @@
 local id = require "prosody.util.id";
 local it = require "prosody.util.iterators";
+local usermanager = require "prosody.core.usermanager";
 local url = require "socket.url";
 local jid_node = require "prosody.util.jid".node;
 local jid_split = require "prosody.util.jid".split;
@@ -221,7 +222,11 @@ local function add_landing_url(invite)
 	-- Determine whether this type of invitation is supported by the landing page
 	local invite_type;
 	if invite.type == "register" then
-		invite_type = "account";
+		if invite.additional_data and invite.additional_data.allow_reset then
+			invite_type = "password-reset";
+		else
+			invite_type = "account";
+		end
 	elseif invite.type == "roster" then
 		if invite.allow_registration then
 			invite_type = "account-and-contact";
@@ -256,6 +261,7 @@ module:add_item("shell-command", {
 	flags = {
 		array_params = { role = true, group = have_group_invites };
 		value_params = { expires_after = true };
+		kv_params = { admin = true };
 	};
 
 	handler = function (self, user_jid, opts) --luacheck: ignore 212/self
@@ -297,10 +303,13 @@ module:add_item("shell-command", {
 	};
 
 	handler = function (self, user_jid, opts) --luacheck: ignore 212/self
-		local username = jid_split(user_jid);
+		local username, host = jid_split(user_jid);
 		if not username then
 			return nil, "Supply the JID of the account you want to generate a password reset for";
 		end
+		if not usermanager.user_exists(username, host) then
+			return nil, "The specified user account does not exist: " .. username;
+		end
 		local duration_sec = require "prosody.util.human.io".parse_duration(opts and opts.expires_after or "1d");
 		if not duration_sec then
 			return nil, "Unable to parse duration: "..opts.expires_after;
